from dbnd import parameter
from dbnd._core.constants import ClusterPolicy, EmrClient, SparkClusters
from dbnd._core.errors import DatabandConfigError, DatabandSystemError
from dbnd._core.plugin.dbnd_plugins import assert_airflow_enabled, is_airflow_enabled
from dbnd._core.task.config import Config


class EmrConfig(Config):
    """Amazon Elastic MapReduce"""

    _conf__task_family = "emr"

    cluster_type = SparkClusters.emr
    cluster = parameter.c(description="Cluster name")[str]

    policy = parameter.c(
        default=ClusterPolicy.NONE, description="Cluster start/stop policy"
    ).choices(ClusterPolicy.ALL)

    region = parameter.c(
        description="region to use for aws connection", default="us-east-2"
    )[str]

    conn_id = parameter.value(
        default="spark_emr", description="spark emr connection settings"
    )

    client = parameter.c(
        default=EmrClient.STEP, description="Type of client used to run EMR jobs"
    )
    ssh_mode = parameter.c(
        description="If ssh mode is on - we'll use localhost:SSH_PORT(8998) to connect to livy",
        default=False,
    )[bool]
    livy_port = parameter[int].c(description="Port to connect to livy", default=8998)

    def _validate(self):
        super(EmrConfig, self)._validate()
        if not self.cluster:
            raise DatabandConfigError(
                "Spark cluster name is not defined! Use --set emr.cluster <CLUSTERNAME> "
            )

    def get_spark_ctrl(self, task_run):

        from dbnd._core.constants import EmrClient

        if self.client == EmrClient.LIVY:
            from dbnd_aws.emr.emr_ctrl import EmrLivyCtr

            return EmrLivyCtr(task_run=task_run)
        else:
            assert_airflow_enabled()
            from dbnd_aws.emr.emr_step import EmrStepCtrl

            return EmrStepCtrl(task_run=task_run)



import boto3

import requests


def get_current_region():
    """
    works only from ec2 machine
    :return:
    """
    r = requests.get("http://169.254.169.254/latest/dynamic/instance-identity/document")
    response_json = r.json()
    return response_json.get("region")


def get_boto_emr_client(region_name=None):
    return boto3.client("emr", region_name=region_name)



from os import environ

from dbnd import parameter
from dbnd._core.constants import CloudType
from dbnd._core.plugin.dbnd_plugins import assert_airflow_enabled
from dbnd._core.settings import EnvConfig


class AwsEnvConfig(EnvConfig):
    """Amazon Web Services"""

    _conf__task_family = CloudType.aws

    conn_id = parameter(
        description="connection id of AWS credentials / region name. "
        "If None,credential boto3 strategy will be used "
        "(http://boto3.readthedocs.io/en/latest/guide/configuration.html)."
    ).value("aws_default")

    region_name = parameter(
        description="region name to use in AWS Hook. "
        "Override the region_name in connection (if provided)"
    ).none[str]

    def prepare_env(self):
        """
        This allows us to use pandas to load remote dataframes directly
        """

        assert_airflow_enabled()

        from dbnd_aws.credentials_helper_aws import AwsCredentials

        boto_session = AwsCredentials(self.conn_id).get_credentials(self.region_name)[0]
        creds = boto_session.get_credentials()

        access_key_env = "AWS_ACCESS_KEY_ID"
        secret_key_env = "AWS_SECRET_ACCESS_KEY"
        if (
            creds.access_key
            and creds.secret_key
            and access_key_env not in environ
            and secret_key_env not in environ
        ):
            environ[access_key_env] = creds.access_key
            environ[secret_key_env] = creds.secret_key



_aws_s3 = None


def _cached_s3_resource():
    global _aws_s3
    if _aws_s3:
        return _aws_s3
    from dbnd_aws.credentials_helper_aws import AwsCredentials

    _aws_s3 = AwsCredentials().get_s3_resource()
    return _aws_s3


def build_s3_fs_client():
    from dbnd_aws.fs.s3 import S3Client

    return S3Client.from_boto_resource(resource=_cached_s3_resource())
